/* $Id $
	------------------------------------------------------------------
	Copyright (C) 2012 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
*/
'use strict';

var Configurations = function()
{
	var configurations = {};
	
	/**
		Initialize the page.
	*/
	configurations.initPage = function()
	{
		var pageId = Doc.getPageId();		
		if (pageId == 'edit-page')
		{
			// Save + Close buttons
			Buttons.addClickHandler('btnSave', configurations.save);
			Buttons.addClickHandler('btnSaveAndConfigure', configurations.saveAndConfigure);
			Buttons.addClickHandler('close', App.closeWindow);

			// Tab validation
			TabControl.addTabValidator('settings.info', configurations.validateConfiguration);
			if (Doc.element('settings.annotations'))
			{
				TabControl.addTabActivateListener('settings.annotations', Annotations.onSwitchToAnnotationsTab);
			}

			// Select plugin
			Buttons.addClickHandler('plugindefinition_id.select', configurations.selectPluginOnClick);
			Events.addEventHandler('plugindefinition_id', 'base-selected', Items.onItemSelected);
		}
		else if (pageId == 'view-page')
		{
			var itemId = Data.get('page-data', 'item-id');
			var attributes = {'item-type': 'PLUGINCONFIGURATION', 'item-id': itemId};
			
			Buttons.addClickHandler('btnEdit', Buttons.editItem, attributes);
			Buttons.addClickHandler('btnDelete', Buttons.deleteItem, attributes);
			Buttons.addClickHandler('btnRestore', Buttons.restoreItem, attributes);
			Buttons.addClickHandler('btnUsingItems', Buttons.showUsingItems, attributes);
			Buttons.addClickHandler('btnDeletePermanently', Buttons.deleteItemPermanently, attributes);
			Buttons.addClickHandler('btnShare', Buttons.shareItem, attributes);
			Buttons.addClickHandler('btnSetOwner', Buttons.setOwner, attributes);
			Buttons.addClickHandler('btnExport', Buttons.runPlugin, attributes);
			Buttons.addClickHandler('btnImport', Buttons.runPlugin, attributes);
			Buttons.addClickHandler('btnRunPlugin', Buttons.runPlugin, attributes);
			Buttons.addClickHandler('btnCopy', configurations.copyConfig, attributes);
			Buttons.addClickHandler('btnConfigure', configurations.configurePlugin, attributes);
			
			if (Doc.element('main.annotations'))
			{
				TabControl.addTabActivateListener('main.annotations', AnnotationsList.loadOnce);
			}
			TabControl.addTabActivateListener('main.history', History.loadOnce);
		}
		else if (pageId == 'list-page')
		{
			var attributes = {'item-type': 'PLUGINCONFIGURATION'};
			var tableAttributes = {'table-id': 'configurations'};
			Buttons.addClickHandler('btnNewItem', Buttons.newItem, attributes);
			Buttons.addClickHandler('btnDeleteItems', Buttons.deleteItems, tableAttributes);
			Buttons.addClickHandler('btnRestoreItems', Buttons.restoreItems, tableAttributes);
			Buttons.addClickHandler('btnShareItems', Buttons.shareItems, tableAttributes);
			Buttons.addClickHandler('btnSetOwner', Buttons.setOwnerOfItems, tableAttributes);
			Buttons.addClickHandler('btnColumns', Buttons.configureColumns, tableAttributes);
			Buttons.addClickHandler('btnExport', Buttons.runListPlugin, tableAttributes);
			Buttons.addClickHandler('btnImport', Buttons.runListPlugin, tableAttributes);
			Buttons.addClickHandler('btnRunPlugin', Buttons.runListPlugin, tableAttributes);
			
			Buttons.addClickHandler('close', App.closeWindow);
			Buttons.addClickHandler('btnOk', Buttons.returnSelected, tableAttributes);
		}
	}


	// Add event handlers to the 'configure' and copy icons
	configurations.initElements = function(element, autoInit)
	{
		if (autoInit == 'configure-plugin')
		{
			Buttons.addClickHandler(element, configurations.configurePlugin);
		}
		else if (autoInit == 'copy-config')
		{
			Buttons.addClickHandler(element, configurations.copyConfig);
		}
	}
	
	/*
		Run the plug-in configuration wizard
	*/
	configurations.configurePlugin = function(event)
	{
		var configId = Data.int(event.currentTarget, 'item-id');
		var url = App.getRoot() + 'common/plugin/index.jsp?ID='+App.getSessionId();
		url += '&cmd=ConfigurePlugin';
		url += '&pluginconfiguration_id='+configId;
		Dialogs.openPopup(url, 'ConfigurePlugin', 750, 500);
	}

	/*
		Create a copy of a configuration
	*/
	configurations.copyConfig = function(event)
	{
		var configId = Data.int(event.currentTarget, 'item-id');
		Items.newItem('PLUGINCONFIGURATION', '&clone_id='+configId);
	}

	configurations.validateConfiguration = function()
	{
		var frm = document.forms['configuration'];
		if (Strings.trim(frm.name.value) == '')
		{
			Forms.showNotification(frm.name, 'You must enter a name');
			return false;
		}
		else if (frm.plugindefinition_id && !frm.plugindefinition_id.value)
		{
			Forms.showNotification('plugindefinition_id.select', 'You must select a plugin');
			return false;
		}
		return true;
	}

	// Submit the form
	configurations.save = function()
	{
		var frm = document.forms['configuration'];
		if (TabControl.validateActiveTab('settings'))
		{
			if (window.Annotations)
			{
				Annotations.saveModifiedAnnotationsToForm(frm);
			}
			frm.submit();
		}
	}
	
	configurations.saveAndConfigure = function()
	{
		var frm = document.forms['configuration'];
		frm.configure.value = 1;
		configurations.save();
	}

	configurations.selectPluginOnClick = function()
	{
		var frm = document.forms['configuration'];
		var url = '&resetTemporary=1&tmpfilter:BOOLEAN:supportsConfigurations=1';

		var mainType = Data.get('plugindefinition_id', 'plugin-main-type');
		if (mainType)
		{
			url += '&tmpfilter:INT:mainType='+mainType;
		}
		var itemType = Data.get('plugindefinition_id', 'plugin-item-type');
		if (itemType)
		{
			url += '&tmpfilter:INT:$ctx.itemType='+itemType;
		}

		if (frm.plugindefinition_id.length > 0) 
		{
			var id = Math.abs(parseInt(frm.plugindefinition_id[0].value));
			url += '&item_id='+frm.plugindefinition_id[0].value;
		}
		Dialogs.selectItem('PLUGINDEFINITION', 'plugindefinition_id', 0, url);
	}
	
	
	return configurations;
}();

Doc.addElementInitializer(Configurations.initElements);
Doc.onLoad(Configurations.initPage);
