/* $Id $
	------------------------------------------------------------------
	Copyright (C) 2012 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
*/
'use strict';

var Inherit = function()
{
	var inherit = {};
	var joustTree;

	inherit.initPage = function()
	{
		// Buttons (on standalone dialog)
		Buttons.addClickHandler('close', App.closeWindow);
		Buttons.addClickHandler('btnInherit', inherit.save);
		Buttons.addClickHandler('btnClone', inherit.save);
		
		Events.addEventHandler('quickFilter', 'keyup', inherit.quickFilter);
	
		IconStore.addDefaultIcons();
		IconStore.addIconSet('Annotatable', 'item.png');
		IconStore.addIconSet('Annotation', 'annotation.png');
		IconStore.addIconSet('Parameter', 'parameter.png');
		IconStore.addIconSet('NonParent', 'item-error.png');

		Events.addEventHandler('joust', 'joust-select', inherit.onSelect);
		Joust2.draw('joust');
		joustTree = Data.json('joust', 'joust-tree');
	}

	/**
		Display annotation values in the right frame.
	*/
	inherit.onSelect = function(event)
	{
		var frm = document.forms['annotations'];
		var menuItem = event.target.item;
		if (menuItem.type == 'annotation')
		{
			Doc.element('annotationType').innerHTML = menuItem.annotationType;
			Doc.element('annotationValues').innerHTML = menuItem.values;
			menuItem.modified = menuItem.inherited != frm[menuItem.id].checked;
		}
		else if (menuItem.type == 'annotation-set')
		{
			if (menuItem.children)
			{
				var checked = frm[menuItem.id].checked;
				// If the annotation-set is checked all child items should
				// be checked and disabled
				// If the annotation-set is not checked all child items should
				// be enabled and check if they are currently inherited
				for (var childNo = 0; childNo < menuItem.children.length; childNo++)
				{
					var child = menuItem.children[childNo];
					frm[child.id].checked = checked || (child.inherited != child.modified);
					frm[child.id].disabled = checked;
				}
			}
		}
		
	}
	
	inherit.saveInheritedAnnotations = function(clone)
	{
		var tree = Doc.element('joust');
		var target = window.top.opener.document.getElementById(Data.get('page-data', 'callback'));
		
		var frm = document.forms['annotations'];
		var menuElements = tree.getElementsByClassName('joustitem');
		for (var menuNo = 0; menuNo < menuElements.length; menuNo++)
		{
			var menuElement = menuElements[menuNo];
			var menuItem = menuElement.item;
			
			var wasInherited = menuItem.inherited;
			var isInherited = frm[menuItem.id].checked;
			if (wasInherited != isInherited)
			{
				if (menuItem.type == 'annotation')
				{
					var detail = {};
					detail.id = menuItem.externalId;
					detail.clone = clone;
					if (isInherited) 
					{
						Events.sendCustomEvent(target, 'inherit-annotation', detail);
					}
					else
					{
						Events.sendCustomEvent(target, 'remove-annotation', detail);	
					}
				}
			}
		}
	}

	inherit.quickFilter = function()
	{
		var frm = document.forms['annotations'];
		var filter = frm.quickFilter.value.toLowerCase();
		var totalVisible = 0;
		for (var itemNo = 0; itemNo < joustTree.length; itemNo++)
		{
			var menuItem = joustTree[itemNo];
			var childrenVisible = 0;
			for (var childNo = 0; childNo < menuItem.children.length; childNo++)
			{
				var childItem = menuItem.children[childNo];
				var visible = !filter || childItem.annotationType.toLowerCase().indexOf(filter)>=0;
				Doc.showHide(childItem.id, visible);
				if (visible) childrenVisible++;
			}
			totalVisible += childrenVisible;
			Doc.showHide(menuItem.id, childrenVisible > 0);
		}

		Doc.showHide('noMatchingFilter', totalVisible == 0);
	}
		
		
	/**
		Save the annotations in standalone mode.
	*/
	inherit.save = function(event)
	{
		var clone = Data.int(event.currentTarget, 'clone', 0);
		inherit.saveInheritedAnnotations(clone);
		App.closeWindow();
	}

	return inherit;
}();

Doc.onLoad(Inherit.initPage);


