/* $Id $
	------------------------------------------------------------------
	Copyright (C) 2013 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
*/
'use strict';

var Configure = function()
{
	var configure = {};
	var parameters;
	var parametersByName = [];
	var selectedParameter = null;
	var selectedValueIndex;
	
	/**
		Initialize the page.
	*/
	configure.initPage = function()
	{
		// Dialog buttons
		Buttons.addClickHandler('close', configure.cancelWizard);
		Buttons.addClickHandler('btnTestWithFile', configure.testWithFile);
		Buttons.addClickHandler('btnNext', configure.saveSettings);
		
		Buttons.addClickHandler('btnShowErrorList', configure.showErrorList);
		Buttons.addClickHandler('btnHideErrorList', configure.hideErrorList);
		
		Buttons.addClickHandler('btnAddValue', configure.addValueOnClick);
		Buttons.addClickHandler('btnRemoveValue', configure.removeValueOnClick);
		Events.addEventHandler('values', 'click', configure.valuesOnClick);
		
		// Parameters
		parameters = Data.json('page-data', 'plugin-parameters');
		var parameterList = Doc.element('parameter-list');
		for (var i = 0; i < parameters.length; i++)
		{
			var pp = parameters[i];
			parametersByName[pp.name] = pp;

			App.debug(JSON.stringify(pp));
			
			// Create entry in parameter selection list
			if (!pp.hidden)
			{
				var pDiv = document.createElement('div');
				pDiv.id = 'parameter-'+pp.name+':prompt';
				Data.set(pDiv, 'parameter-name', pp.name);
				
				if (!pp.valueClass)
				{
					// Section header entry
					pDiv.className = 'section interactable';
					pDiv.innerHTML = Strings.encodeTags(pp.label);
				}
				else
				{
					// Parameter entry
					pDiv.className = 'param interactable';
					var icon = '';
					if (pp.values.length > 0)
					{
						icon = pp.nullable ? 'notrequired_values.png' : 'required_values.png';	
					}
					else
					{
						icon = pp.nullable ? 'notrequired_novalues.png' : 'required_novalues.png';
					}
						
					var html = '<span class="icon">';
					html += '<img id="parameter-'+pp.name+':icon" src="../../images/'+icon+'" alt="">';
					html += '</span>';
					html += '<span class="label">'+Strings.encodeTags(pp.label)+'</span>';
					pDiv.innerHTML = html;
				}
				parameterList.appendChild(pDiv);
				Events.addEventHandler(pDiv, 'click', configure.parameterOnClick);
			}
			
			// Attach event handlers to input fields
			var valueClass = pp.valueClass;
			var fieldName = 'parameter-'+pp.name;
			// Detect changes
			if (valueClass != 'BooleanParameterType')
			{
				Events.addEventHandler(fieldName, 'change', configure.parameterValueOnChanage);
			}
			else
			{
				Events.addEventHandler(fieldName+':null', 'change', configure.parameterValueOnChanage);
				Events.addEventHandler(fieldName+':true', 'change', configure.parameterValueOnChanage);
				Events.addEventHandler(fieldName+':false', 'change', configure.parameterValueOnChanage);
			}
			if (valueClass == 'IntegerParameterType' || valueClass == 'LongParameterType')
			{
				// Block non-numeric keys from INT and LONG fields
				Events.addEventHandler(fieldName, 'keypress', Events.integerOnly);
			}
			else if (valueClass == 'FloatParameterType' || valueClass == 'DoubleParameterType')
			{
				// Block non-numeric keys from FLOAT and DOUBLE fields
				Events.addEventHandler(fieldName, 'keypress', Events.numberOnly);
			}
			else if (valueClass == 'FileParameterType')
			{
				// Browse button
				Buttons.addClickHandler(fieldName+':browse', configure.selectFileOnClick);
				Events.addEventHandler(fieldName, 'base-selected', configure.onFileSelected);
				Events.addEventHandler(fieldName+':recent', 'change', configure.recentFileSelected);
			}
			else if (valueClass = 'PathParameterType')
			{
				// Browse button
				if (pp.pathType == 'FILE')
				{
					Buttons.addClickHandler(fieldName+':browse', configure.saveAsOnClick);
					Events.addEventHandler(fieldName, 'base-selected', configure.onSaveAsSelected);
				}
				else
				{
					Buttons.addClickHandler(fieldName+':browse', configure.selectDirectoryOnClick);
					Events.addEventHandler(fieldName, 'base-selected', configure.onFileSelected);
				}
			}
		}
		
	}
	
	configure.initElements = function(element, autoInit)
	{
		if (autoInit == 'toggle-stacktrace')
		{
			Buttons.addClickHandler(element, configure.toggleStacktrace);
		}
		else if (autoInit == 'select-all')
		{
			Buttons.addClickHandler(element, configure.selectAll);
		}
	}
	
	/**
		When clicking on a parameter in the selection list, the input fields
		for that section should be displayed.
	*/
	configure.parameterOnClick = function(event)
	{
		var name = Data.get(event.currentTarget, 'parameter-name');
		var parameter = parametersByName[name];
		if (parameter) configure.selectParameter(parameter);
	}
	
	configure.selectParameter = function(parameter)
	{
		if (selectedParameter)
		{
			// De-select current parameter
			Doc.removeClass('parameter-'+selectedParameter.name+':prompt', 'selected');
			Doc.hide('parameter-'+selectedParameter.name+':section');
			Doc.hide('valuecontainer');
		}
		
		selectedParameter = parameter;
		
		Doc.addClass('parameter-'+parameter.name+':prompt', 'selected');
		Doc.show('parameter-'+parameter.name+':section');
		
		var frm = document.forms['configure'];
		var field = frm['parameter-'+parameter.name];
		if (field.focus && !field.disabled) field.focus();

		if (parameter.enumeration)
		{
			// Enumerated parameters are in a <select> element
			var numSelected = Forms.selectListOptions(field, parameter.values, true);
			if (numSelected == 0)
			{
				parameter.values.length = 0;
				configure.updateStyle(parameter);
			}
		}
		else if (parameter.multiplicity == 1)
		{
			// Single-values parameters are 'text' or 'radio' elements
			var currentValue = parameter.values.length > 0 ? parameter.values[0] : '';
			if (field.length) // radio buttons for boolean parameter
			{
				Forms.checkRadio(field, currentValue);
			}
			else if (!field.disabled)
			{
				field.value = currentValue;
			}
		}
		else
		{
			// Multi-values parameters must be copied to a <select> element
			var values = frm.values;
			// Clear existing values from the <select> element
			values.length = 0;
			// Copy values
			for (var i = 0; i < parameter.values.length; i++)
			{
				var value = Strings.cut(parameter.values[i].replace(/\n/g, ' '), 40);
				values[values.length] = new Option(value);
			}
			var mult = Doc.element('multiplicity');
			mult.innerHTML = parameter.multiplicity == 0 ? '' : '(Max '+parameter.multiplicity+' values)';
			Doc.show('valuecontainer');
		}
		
	}

	
	configure.parameterValueOnChanage = function(event)
	{
		var frm = document.forms['configure'];
		if (!selectedParameter) return;
		
		if (selectedParameter.multiplicity == 1)
		{
			// Single-valued parameters are copied as a new array
			var value = event.target.value;
			if (value == null || value == '')
			{
				selectedParameter.values = [];
			}
			else
			{
				selectedParameter.values = [value];
			}
		}
		else if (selectedParameter.enumeration)
		{
			// All values are in a <select multiple> element
			var values = [];
			for (var i = 0; i < event.target.length; i++)
			{
				if (event.target[i].checked || event.target[i].selected) 
				{
					values[values.length] = event.target[i].value;
				}
			}
			selectedParameter.values = values;
		}
		else
		{
			// Multi-valued parameters are updated for the current index
			if (selectedValueIndex >= 0)
			{
				var value = event.target.value;
				if (value == null || value == '')
				{
					selectedParameter.values.splice(selectedValueIndex, 1);
					frm.values[selectedValueIndex] = null;
				}
				else
				{
					selectedParameter.values[selectedValueIndex] = value;
					frm.values[selectedValueIndex].text = value;
				}
			}
		}
		
		configure.updateStyle(selectedParameter);
	}
	
	configure.updateStyle = function(parameter)
	{
		var icon = Doc.element('parameter-'+parameter.name+':icon');
		var img = App.getRoot()+'images/';
		if (parameter.values.length == 0)
		{
			img += parameter.nullable ? 'notrequired_novalues.png' : 'required_novalues.png';
		}
		else
		{
			img += parameter.nullable ? 'notrequired_values.png' : 'required_values.png';
		}
		icon.src = img;
	}

	
	configure.selectFileOnClick = function(event)
	{
		Dialogs.selectItem('FILE', Data.get(event.currentTarget, 'field'));
	}
	
	configure.onFileSelected = function(event)
	{
		event.currentTarget.value = event.detail.name;
		Events.sendChangeEvent(event.target);
		if (parametersByName['charset'])
		{
			configure.getCharsetOfFile(event.detail.id);
		}
	}
	
	configure.recentFileSelected = function(event)
	{
		var fileList = event.currentTarget;
		var field = Data.get(fileList, 'field');
		var detail = {'id': fileList.value, 'name': fileList[fileList.selectedIndex].text};
		Events.sendCustomEvent(field, 'base-selected', detail);
		fileList.selectedIndex = 0;
	}

	configure.saveAsOnClick = function(event)
	{
		var field = Data.get(event.currentTarget, 'field');
		var url = App.getRoot() +'filemanager/index.jsp?ID='+App.getSessionId();
		url += '&cmd=SaveAs&callback='+field;

		var frm = document.forms['configure'];
		var i = frm[field].value.lastIndexOf('/');
		if (i >= 0) 
		{
			url += '&fileName=' + encodeURIComponent(frm[field].value.substring(i+1));
		}

		Dialogs.openPopup(url, 'SaveAs', 1050, 700);
	}
	
	configure.onSaveAsSelected = function(event)
	{
		event.target.value = event.detail.path;
		Events.sendChangeEvent(event.target);
	}
	
	configure.selectDirectoryOnClick = function(event)
	{
		Dialogs.selectItem('DIRECTORY', Data.get(event.currentTarget, 'field'));
	}
	
	configure.getCharsetOfFile = function(fileId)
	{
		var request = Ajax.getXmlHttpRequest();
		var url = '../../filemanager/files/ajax.jsp?ID='+App.getSessionId();
		url += '&cmd=GetFileInfo&item_id=' + fileId;
		request.open("GET", url, true);
		Ajax.setReadyStateHandler(request, configure.updateCharsetCallback);
		request.send(null);
	}
	
	configure.updateCharsetCallback = function(request)
	{
		var response = JSON.parse(request.responseText);
		var charset = response.characterSet;
		if (!charset) return;
		
		var parameter = parametersByName['charset'];
		if (!parameter) return;
		
		parameter.values = [ charset ];
		configure.updateStyle(parameter);
	}
	
	configure.selectAll = function(event)
	{
		var field = Data.get(event.currentTarget, 'field');
		var frm = document.forms['configure'];
		var list = Doc.element(field);
		
		if (!list || list.length == 0) return;
		var select = !list[0].selected;;
		
		for (var i = 0; i < list.length; i++)
		{
			list[i].selected = select;
		}
		Events.sendChangeEvent(list);
	}
	
	configure.cancelWizard = function()
	{
		location.href = 'index.jsp?ID='+App.getSessionId()+'&cmd=CancelWizard';
	}
	
	configure.testWithFile = function()
	{
		var url = 'test_with_file.jsp?ID='+App.getSessionId();
		Dialogs.openPopup(url, 'TestWithFile', 1050, 700);
	}
	
	configure.valuesOnClick = function()
	{
		var frm = document.forms['configure'];
		if (!selectedParameter) return;
		
		selectedValueIndex = frm.values.selectedIndex;
		if (selectedValueIndex >= 0)
		{
			frm['parameter-'+selectedParameter.name].value = selectedParameter.values[selectedValueIndex];
		}
	}
	
	configure.addValueOnClick = function()
	{
		if (!selectedParameter) return;

		// Check multiplicity
		var frm = document.forms['configure'];
		var field = frm['parameter-'+selectedParameter.name];
		if (selectedParameter.multiplicity > 0 && selectedParameter.values.length >= selectedParameter.multiplicity)
		{
			Forms.showNotification('btnAddValue', 'Max '+selectedParameter.multiplicity+' values allowed');
			return;
		}
		
		if (frm.values.selectedIndex < 0 && field.value != '')
		{
			// Insert a new option with the value from the field
			frm.values[frm.values.length] = new Option(field.value, '');
			selectedParameter.values[selectedParameter.values.length] = field.value;
			field.value = '';
			selectedValueIndex = -1;
		}
		else
		{
			// Clear current selection
			for (var i = 0; i < frm.values.length; i++)
			{
				frm.values[i].selected = false;
			}
			// Insert an empty option
			selectedValueIndex = frm.values.length;
			frm.values[selectedValueIndex] = new Option('<new>', '', false, true);
			selectedParameter.values[selectedValueIndex] = '';
			field.value = '';
		}
		field.focus();
		configure.updateStyle(selectedParameter);
	}
	
	configure.removeValueOnClick = function()
	{
		if (!selectedParameter) return;
		
		var frm = document.forms['configure'];
		var values = frm.values;
		for (var i = 0; i < values.length; i++)
		{
			if (values[i].selected)
			{
				selectedParameter.values.splice(i, 1);
				values[i] = null;
				i--;
			}
		}
		selectedValueIndex = -1;
		frm.values.selectedIndex = -1;
		frm['parameter-'+selectedParameter.name].value = '';
		configure.updateStyle(selectedParameter);
	}


	configure.saveSettings = function()
	{
		var frm = document.forms['configure'];
		
		// Check that all required parameters have values
		for (var i = 0; i < parameters.length; i++)
		{
			var pp = parameters[i];
			if (pp.valueClass)
			{
				if (!pp.nullable && pp.values.length == 0)
				{
					configure.selectParameter(pp);
					Forms.showNotification(frm['parameter-'+pp.name], 'No value entered for parameter '+pp.label);
					return;
				}
			}
		}

		// Save parameters to form and submit
		for (var i = 0; i < parameters.length; i++)
		{
			var pp = parameters[i];
			if (pp.valueClass)
			{
				for (var j = 0; j < pp.values.length; j++)
				{
					Forms.addHidden(frm, 'parameter:'+pp.name, pp.values[j]);
				}
			}
		}

		frm.submit();
	}
	
	configure.hideErrorList = function()
	{
		Doc.hide('errorlist');
		Doc.show('showerrorlist');
	}
	configure.showErrorList = function()
	{
		Doc.show('errorlist');
		Doc.hide('showerrorlist');
	}

	configure.toggleStacktrace = function(event)
	{
		var index = Data.get(event.currentTarget, 'stracktrace-index');
		Doc.showHide('stacktrace.' + index);
		var img = event.currentTarget.firstChild;
		if (img.src.indexOf('gonext') > 0)
		{
			img.src = App.getRoot() + '/images/move_down.png';
		}
		else
		{
			img.src = App.getRoot() + '/images/gonext.png';
		}
	}

	configure.getParameterDef = function(name)
	{
		return parametersByName[name];
	}
	
	configure.getParameterValue = function(name)
	{
		var parameter = parametersByName[name];
		if (parameter && parameter.values.length > 0)
		{
			return parameter.values[0];
		}
		else
		{
			return '';
		}
	}
	
	configure.getAllParameters = function()
	{
		return parameters;
	}
	
	configure.setParameterValue = function(name, value)
	{
		var parameter = parametersByName[name];
		if (!parameter) return;
		if (value == null || value == '')
		{
			parameter.values = [];
		}
		else
		{
			parameter.values = [value];
		}
		configure.updateStyle(parameter);
		if (selectedParameter && selectedParameter.name == name)
		{
			configure.selectParameter(parameter);
		}
	}

	return configure;
}();

Doc.onLoad(Configure.initPage);
Doc.addElementInitializer(Configure.initElements);