/* $Id$
	------------------------------------------------------------------
	Copyright (C) 2011 Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
	@version 2.0
*/
'use strict';

var DragDrop = function()
{
	var dragdrop = {};
	var internal = {};

	var dragElement = null;
	var startX = 0;
	var startY = 0;
	
	/**
		Enable dragging on the given element. Dragging can be
		enabled independently in the horizontal or vertical direction.
		The callback function should be a function that takes two parameters;
		a delta-X and delta-Y which is the number of pixels the mouse has
		moved since the last call to the callback or since the drag started.
	*/
	dragdrop.enableDrag = function(element, horizontal, vertical, callback)
	{
		element = Doc.element(element);
		element.dragHorizontal = horizontal;
		element.dragVertical = vertical;
		element.dragCallback = callback;
		Events.addEventHandler(element, 'mousedown', dragdrop.beginDrag);
	}
	
	/**
		Start dragging. The 'event' parameter is used to get
		the mouse position. The callback function is called
		when the mouse is moved to handle the drag visually.
	*/
	dragdrop.beginDrag = function(event)
	{
		startX = event.screenX;
		startY = event.screenY;
		dragElement = event.currentTarget;
	}

	/**
		Recieve mouse move events. Calculate the delta X and Y since the
		last position and call the callback function.
	*/
	dragdrop.drag = function(event)
	{
		if (!dragElement) return;
		var deltaX = event.screenX - startX;
		var deltaY = -(event.screenY - startY);
		
		if ((dragElement.dragHorizontal && Math.abs(deltaX) > 0) || (dragElement.dragVertical && Math.abs(deltaY) > 0)) 
		{
			dragElement.dragCallback.call(dragElement, deltaX, deltaY);
			startX = event.screenX;
			startY = event.screenY;
		}
	}

	/**
		Stop dragging.
	*/
	dragdrop.endDrag = function(event)
	{
		dragElement = null;
	}
	
	internal.initDragSupport = function(element, autoInit)
	{
		if (autoInit != 'drag-support') return;
		var dd = parent.DragDrop ? parent.DragDrop : dragdrop;
		Events.addEventHandler(element, 'mousemove', dd.drag);
		Events.addEventHandler(element, 'mouseup', dd.endDrag);
	}
	Doc.addElementInitializer(internal.initDragSupport);
	
	return dragdrop;
}();

