<%-- $Id: index.jsp 7982 2021-06-14 08:01:21Z nicklas $
	------------------------------------------------------------------
	Copyright (C) 2006 Jari Häkkinen, Nicklas Nordborg

	This file is part of BASE - BioArray Software Environment.
	Available at http://base.thep.lu.se/

	BASE is free software; you can redistribute it and/or
	modify it under the terms of the GNU General Public License
	as published by the Free Software Foundation; either version 3
	of the License, or (at your option) any later version.

	BASE is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with BASE. If not, see <http://www.gnu.org/licenses/>.
	------------------------------------------------------------------

	@author Nicklas
	@version 2.0
--%>
<%@ page pageEncoding="UTF-8" session="false"
	import="net.sf.basedb.core.SessionControl"
	import="net.sf.basedb.core.DbControl"
	import="net.sf.basedb.core.Item"
	import="net.sf.basedb.core.BioAssaySet"
	import="net.sf.basedb.core.BioAssay"
	import="net.sf.basedb.core.RawDataType"
	import="net.sf.basedb.core.RawDataTypes"
	import="net.sf.basedb.core.ItemContext"
	import="net.sf.basedb.core.query.SqlQuery"
	import="net.sf.basedb.clients.web.Base"
	import="net.sf.basedb.clients.web.WebException"
	import="net.sf.basedb.util.Values"
	import="net.sf.basedb.util.formatter.LookupFormatter"
	import="net.sf.basedb.clients.web.util.HTML"
	import="java.util.Map"
	import="java.util.HashMap"
	import="java.util.Collections"
%>
<%@ taglib prefix="base" uri="/WEB-INF/base.tld" %>
<%!
	private static final Map<RawDataType, ItemContext> defaultContexts = 
		new HashMap<RawDataType, ItemContext>();

	static
	{
		for (RawDataType rdt : RawDataTypes.getRawDataTypes())
		{
			// Default visible columns are: position, ch1, ch2, ...
			StringBuilder columns = new StringBuilder("POSITION,rep.externalId");
			for (int ch = 1; ch <= rdt.getChannels(); ++ch)
			{
				columns.append(",ch").append(ch);
			}
			defaultContexts.put(rdt, 
				Base.createDefaultContext("POSITION", columns.toString(), "rep.externalId"));
		}
	}
	private static final Item itemType = Item.SPOTDATA;
	
	private static void registerExportFormatters(DbControl dc, ItemContext cc, 
		BioAssaySet bas, BioAssay ba)
	{
		if (bas != null)
		{
			Map<Number, String> bioAssayMap = new HashMap<Number, String>();
			for (BioAssay b : bas.getBioAssays().list(dc))
			{
				short columnNo = b.getDataCubeColumnNo();
				String name = b.getName();
				// Safety measure, since JDBC driver seems to return 'short' as 'int'
				bioAssayMap.put(columnNo, name);
				bioAssayMap.put((int)columnNo, name);
			}
			cc.setObject("export.formatter.COLUMN", new LookupFormatter<>(bioAssayMap));
		}
	}
%>
<%
final int bioAssayId = Values.getInt(request.getParameter("bioassay_id"));
final int bioAssaySetId = Values.getInt(request.getParameter("bioassayset_id"));

final SessionControl sc = Base.getExistingSessionControl(pageContext, true);
final String ID = sc.getId();
final String cmd = request.getParameter("cmd");
final String root = request.getContextPath()+"/";
final String mode = request.getParameter("mode");
final String callback = request.getParameter("callback");
final String itemId = request.getParameter("item_id");
final String listPage = "list_spotdata.jsp?ID="+ID
	+"&bioassay_id="+bioAssayId+"&bioassayset_id="+bioAssaySetId
	+(mode == null ? "" : "&mode="+mode)
	+(callback == null ? "" : "&callback="+callback)
	+(itemId == null ? "" : "&item_id="+itemId);

String forward = null;
String redirect = null;
String message = null;
DbControl dc = null;

try
{
	dc = sc.newDbControl(":Load bioassays");
	final BioAssay ba = bioAssayId == 0 ? null : BioAssay.getById(dc, bioAssayId);
	final BioAssaySet bas = ba != null ? ba.getBioAssaySet() : BioAssaySet.getById(dc, bioAssaySetId);
	dc.close();
	final RawDataType rawDataType = bas.getRawDataType();
	final ItemContext defaultContext = defaultContexts.get(rawDataType);
	final String subContext = rawDataType.getId();
	
	if (cmd == null || "List".equals(cmd))
	{
		// Display the list page without updatinging the current context
		ItemContext cc = Base.getAndSetCurrentContext(sc, itemType, subContext, null, defaultContext);
		redirect = listPage;
	}
	else if ("UpdateContext".equals(cmd))
	{
		// Display the list page after updating the current context from the request parameters
		ItemContext cc = Base.getAndSetCurrentContext(sc, itemType, subContext, pageContext, defaultContext);
		redirect = listPage;
	}
	else if ("LoadContext".equals(cmd))
	{
		// Display the list page after loading a saved context
		int contextId = Values.getInt(request.getParameter("context"));
		Base.loadContext(sc, contextId, defaultContext);
		redirect = listPage;
	}
	else if ("ExportItems".equals(cmd))
	{
		// Run an export plugin in a list context
		ItemContext cc = Base.getAndSetCurrentContext(sc, itemType, subContext, pageContext, defaultContext);
		dc = sc.newDbControl(":Export "+itemType);
		if (bas != null) dc.reattachItem(bas, false);
		if (ba != null) dc.reattachItem(ba, false);
		final SqlQuery query = ba != null ? ba.getSpotData() : bas.getSpotData();
		registerExportFormatters(dc, cc, bas, ba);
		dc.close();
		cc.setQuery(query);
		redirect = "../../../common/export/index.jsp?ID="+ID+"&cmd=SelectPlugin&item_type="+itemType.name()+"&subcontext="+subContext+"&context_type=LIST&title=Export+spot+data";
	}
	else if ("CreateReporterList".equals(cmd))
	{
		ItemContext cc = Base.getAndSetCurrentContext(sc, itemType, subContext, pageContext, defaultContext);
		dc = sc.newDbControl(":Create reporter list");
		if (bas != null) dc.reattachItem(bas, false);
		if (ba != null) dc.reattachItem(ba, false);
		final SqlQuery query = ba != null ? ba.getSpotData() : bas.getSpotData();
		cc.configureQuery(dc, query, Collections.singletonList("@id"));
		dc.close();
		cc.setQuery(query);
		redirect = "../../../views/reporterlists/index.jsp?ID="+ID+
			"&cmd=NewItem&addReporters=1&formId=spotdata&fromContext=SPOTDATA&subContext=" + subContext +
			"&name=" + HTML.urlEncode(ba != null ? ba.getName() : bas.getName());
	}
	
	else
	{
		throw new WebException("popup", "Invalid command", "The command {1} is not recognised as a valid command.", cmd);
	}
}
finally
{
	if (dc != null) dc.close();
}

if (forward != null)
{
	sc.setSessionSetting("alert-message", message);
	pageContext.forward(forward);
}
else if (redirect != null)
{
	sc.setSessionSetting("alert-message", message);
	response.sendRedirect(redirect);
}
else if (message == null)
{
	response.sendRedirect(root + "common/close_popup.jsp?refresh_opener=1&wait=0");
}
else
{
	response.sendRedirect(root + "common/close_popup.jsp?refresh_opener=1&message="+HTML.urlEncode(message));
}
%>

